from packs import *
from classes import loadConfig, Set
import verbosity_setting
import requests
import requests_cache
from pprint import pprint
from io import BytesIO
from PIL import Image, ImageChops
from helpers import scryfallRequest
import verbosity_setting
from ruamel.yaml import YAML
verbosity_setting.verbosity = 0

#Load the data.
yaml = YAML()
with open('config/mystery-booster.yaml') as yamlFile:
	slots = yaml.load(yamlFile)


def find_matching_set(card_name, foil=False):
	if foil:
		set_code = 'fmb1'
	else:
		set_code = 'mb1'
	
	params = {
		'q': f'!"{card_name}" game:paper -is:promo',
		'unique': 'prints',
		'order' : 'released',
	}
	cards = scryfallRequest(params)
	
	if len(cards) == 2 and cards[0]['set'] == set_code:
		#Only in two sets: mystery booster and whater it was previously in.
		return cards[1]['set']
	
	printings = []
	smallest_diff = 256
	closest_match = None
	
	for card in cards:
		response = requests.get(card['image_uris']['normal'])
		image = Image.open(BytesIO(response.content)).crop((420, 380, 460, 420))
		if card['set'] == set_code:
			mb_image = image
		else:
			printings.append((image, card['set']))
	
	for printing in printings:
		diff_image = ImageChops.difference(mb_image, printing[0]).convert('L')
		diff = diff_image.getextrema()[1]
		if diff < smallest_diff:
			smallest_diff = diff
			closest_match = printing[1]
			
	return closest_match

def find_all_sets(slots):
	for slot_name in slots:
		print(slot_name)
		for entry in slots[slot_name]:
			name = entry[0]
			print(f'\t{name}:')
			set_ = find_matching_set(name, foil=slot=='foil')
			print(f'\t\t{set_}')
			entry[1] = set_

	with open ('config/mystery-booster-new.yaml', 'w') as outputFile:
		yaml.dump(slots, outputFile)

def check_for_cards():
	needed_sets = {}
	
	for slot in slots.values():
		for entry in slot:
			name, set_code = entry
			set_ = None
			try:
				set_ = Set[set_code]
			except KeyError:
				print(f'Tried to get card "{name}" from set "{set_code}", but that set isn\'t in the plugin.')
				if needed_sets.get(set_code, False):
					needed_sets[set_code].append(name)
				else:
					needed_sets[set_code] = [name]
			if set_ is not None:
				if not name in [card.name for card in set_.cards]:
					pass
					#print(f'Tried to get card "{name}" from set "{set_code}", but that set doesn\'t have that card.')
	
	if needed_sets:
		pprint(needed_sets)

def add_draft_qualities():
	print('Adding Mystery Booster draft qualities.')
	needed_sets = {}
	
	for slot_name in slots:
		for entry in slots[slot_name]:
			name, set_code = entry
			try:
				set_ = Set[set_code]
			except KeyError:
				print(f'Tried to find card "{name}" from set "{set_code}", but that set isn\'t in the plugin')
			if set_ is not None:
				found = False
				for card in set_.cards:
					if card.name == name:
						card.draftQualities = f'{card.draftQualities} mb1-{slot_name}'
						found = True
						break
				if not found:
					if needed_sets.get(set_code, False):
						needed_sets[set_code].append(name)
					else:
						needed_sets[set_code] = [name]
					#print(f'Tried to get card "{name}" from set "{set_code}", but that set doesn\'t have that card.')
	print('Cards needed for Mystery Booster:')				
	pprint(needed_sets)
							

if __name__ == '__main__':
	requests_cache.install_cache('mb1_cache')
	
	loadConfig()
	check_for_cards()
	



